/*    Copyright (C) 2004  Peter Krantz

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

------------------------------------------------------------------

This program is based on the SimpleTTS demo distributed with the 
Microsoft Speech SDK (SAPI 5.1). This program requires the SAPI
5.1 or later to be installed.

For more information see http://www.standards-schmandards.com

*/
namespace SimpleTTS
{
    using System;
    using System.Windows.Forms;
    using System.Threading;
    using SpeechLib;
	using System.Data;
	using System.IO;
	using System.Collections;

    /// <summary>
    /// Captcha audio generator. For more information see http://www.standards-schmandards.com
    /// </summary>
    public class MainForm : System.Windows.Forms.Form
    {
        private System.Windows.Forms.Button btnSpeak;
        private System.Windows.Forms.TextBox txtSpeakText;
        private System.Windows.Forms.Button btnExit;
		private System.Windows.Forms.ComboBox comboBox1;
		private System.Windows.Forms.TrackBar tbRate;
		private SpeechLib.SpVoice SlVoice;

		private const string Mode1Text = "Here are the %s numbers: ";
		private const string Mode2Text1 = "First number is: ";
		private const string Mode2Text2 = " Second number is: ";
		private const string Mode2Text3 = " Third number is: ";

		private System.Windows.Forms.FolderBrowserDialog folderBrowserDialog1;
		private System.Windows.Forms.Button button1;
		private System.Windows.Forms.Button button2;
		private System.Windows.Forms.TextBox tbTestInstanceCount;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Button btnGenerate;
		private System.Windows.Forms.GroupBox gbTestSettings;
		private System.Windows.Forms.DataGrid dgGeneratedTestInstances;
		
		private System.Windows.Forms.Label lblTargetFolder;
		private System.Windows.Forms.Label label1;
		private const string SelectFolder = "No folder selected";
		private const string UseRandomVoice = "Use random voices";
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.TextBox tbPitchVariation;

		private DataTable dtItems;


        public MainForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

			//Get available voices
			SlVoice = new SpeechLib.SpVoice();

			//Add voices to drop down list
			comboBox1.Items.Add(UseRandomVoice);

			foreach(ISpeechObjectToken sot in SlVoice.GetVoices("",""))
			{
				//Add voice to list
				if(sot.GetDescription(0) != "Sample TTS Voice")
				{
					comboBox1.Items.Add(sot.GetDescription(0));
				}
			}

			//Set default voice
			comboBox1.SelectedIndex = 0;

			//Set text for folder picker label
			lblTargetFolder.Text = SelectFolder;

			//Set up result table
			dtItems = new DataTable("TestInstances");

			// Create columns, and add them to the table
			DataColumn dcFilename = new DataColumn("Filename");
			DataColumn dcFirst = new DataColumn("FirstNo", typeof(int));
			DataColumn dcSecond = new DataColumn("SecondNo", typeof(int));
			DataColumn dcThird = new DataColumn("ThirdNo", typeof(int));
			DataColumn dcText = new DataColumn("Text");
			DataColumn dcVoice = new DataColumn("Voice");
			
			dtItems.Columns.Add(dcFilename);
			dtItems.Columns.Add(dcFirst);
			dtItems.Columns.Add(dcSecond);
			dtItems.Columns.Add(dcThird);
			dtItems.Columns.Add(dcText);
			dtItems.Columns.Add(dcVoice);
        }


        #region Windows Form Designer generated code
        /// <summary>
        ///     Required method for Designer support - do not modify
        ///     the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.btnSpeak = new System.Windows.Forms.Button();
			this.btnExit = new System.Windows.Forms.Button();
			this.txtSpeakText = new System.Windows.Forms.TextBox();
			this.comboBox1 = new System.Windows.Forms.ComboBox();
			this.tbRate = new System.Windows.Forms.TrackBar();
			this.lblTargetFolder = new System.Windows.Forms.Label();
			this.button1 = new System.Windows.Forms.Button();
			this.folderBrowserDialog1 = new System.Windows.Forms.FolderBrowserDialog();
			this.button2 = new System.Windows.Forms.Button();
			this.gbTestSettings = new System.Windows.Forms.GroupBox();
			this.label1 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.tbTestInstanceCount = new System.Windows.Forms.TextBox();
			this.btnGenerate = new System.Windows.Forms.Button();
			this.dgGeneratedTestInstances = new System.Windows.Forms.DataGrid();
			this.label5 = new System.Windows.Forms.Label();
			this.tbPitchVariation = new System.Windows.Forms.TextBox();
			this.label6 = new System.Windows.Forms.Label();
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			((System.ComponentModel.ISupportInitialize)(this.tbRate)).BeginInit();
			this.gbTestSettings.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.dgGeneratedTestInstances)).BeginInit();
			this.SuspendLayout();
			// 
			// btnSpeak
			// 
			this.btnSpeak.Location = new System.Drawing.Point(424, 8);
			this.btnSpeak.Name = "btnSpeak";
			this.btnSpeak.Size = new System.Drawing.Size(48, 24);
			this.btnSpeak.TabIndex = 4;
			this.btnSpeak.Text = "&Speak";
			this.btnSpeak.Click += new System.EventHandler(this.btnSpeak_Click);
			// 
			// btnExit
			// 
			this.btnExit.Location = new System.Drawing.Point(392, 208);
			this.btnExit.Name = "btnExit";
			this.btnExit.Size = new System.Drawing.Size(80, 24);
			this.btnExit.TabIndex = 2;
			this.btnExit.Text = "&Exit";
			this.btnExit.Click += new System.EventHandler(this.btnExit_Click);
			// 
			// txtSpeakText
			// 
			this.txtSpeakText.Location = new System.Drawing.Point(8, 8);
			this.txtSpeakText.Multiline = true;
			this.txtSpeakText.Name = "txtSpeakText";
			this.txtSpeakText.Size = new System.Drawing.Size(284, 24);
			this.txtSpeakText.TabIndex = 3;
			this.txtSpeakText.Text = "";
			// 
			// comboBox1
			// 
			this.comboBox1.Location = new System.Drawing.Point(112, 48);
			this.comboBox1.Name = "comboBox1";
			this.comboBox1.Size = new System.Drawing.Size(144, 21);
			this.comboBox1.TabIndex = 6;
			// 
			// tbRate
			// 
			this.tbRate.Location = new System.Drawing.Point(104, 72);
			this.tbRate.Minimum = -10;
			this.tbRate.Name = "tbRate";
			this.tbRate.Size = new System.Drawing.Size(160, 45);
			this.tbRate.TabIndex = 7;
			// 
			// lblTargetFolder
			// 
			this.lblTargetFolder.Location = new System.Drawing.Point(112, 136);
			this.lblTargetFolder.Name = "lblTargetFolder";
			this.lblTargetFolder.Size = new System.Drawing.Size(256, 16);
			this.lblTargetFolder.TabIndex = 9;
			this.lblTargetFolder.Text = "No folder selected";
			// 
			// button1
			// 
			this.button1.Location = new System.Drawing.Point(376, 128);
			this.button1.Name = "button1";
			this.button1.Size = new System.Drawing.Size(80, 24);
			this.button1.TabIndex = 10;
			this.button1.Text = "Select...";
			this.button1.Click += new System.EventHandler(this.button1_Click);
			// 
			// button2
			// 
			this.button2.Location = new System.Drawing.Point(296, 8);
			this.button2.Name = "button2";
			this.button2.Size = new System.Drawing.Size(120, 24);
			this.button2.TabIndex = 9;
			this.button2.Text = "Create sample text";
			this.button2.Click += new System.EventHandler(this.button2_Click);
			// 
			// gbTestSettings
			// 
			this.gbTestSettings.Controls.Add(this.tbPitchVariation);
			this.gbTestSettings.Controls.Add(this.label6);
			this.gbTestSettings.Controls.Add(this.label5);
			this.gbTestSettings.Controls.Add(this.label1);
			this.gbTestSettings.Controls.Add(this.label4);
			this.gbTestSettings.Controls.Add(this.label3);
			this.gbTestSettings.Controls.Add(this.label2);
			this.gbTestSettings.Controls.Add(this.tbRate);
			this.gbTestSettings.Controls.Add(this.comboBox1);
			this.gbTestSettings.Controls.Add(this.tbTestInstanceCount);
			this.gbTestSettings.Controls.Add(this.button1);
			this.gbTestSettings.Controls.Add(this.lblTargetFolder);
			this.gbTestSettings.Location = new System.Drawing.Point(8, 40);
			this.gbTestSettings.Name = "gbTestSettings";
			this.gbTestSettings.Size = new System.Drawing.Size(464, 160);
			this.gbTestSettings.TabIndex = 10;
			this.gbTestSettings.TabStop = false;
			this.gbTestSettings.Text = "Test settings";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(16, 136);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(88, 16);
			this.label1.TabIndex = 15;
			this.label1.Text = "Output dir:";
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(16, 80);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(80, 24);
			this.label4.TabIndex = 14;
			this.label4.Text = "Pitch:";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(16, 48);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(88, 24);
			this.label3.TabIndex = 13;
			this.label3.Text = "Voice:";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(16, 24);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(96, 23);
			this.label2.TabIndex = 12;
			this.label2.Text = "Tests to generate:";
			// 
			// tbTestInstanceCount
			// 
			this.tbTestInstanceCount.Location = new System.Drawing.Point(112, 24);
			this.tbTestInstanceCount.Name = "tbTestInstanceCount";
			this.tbTestInstanceCount.Size = new System.Drawing.Size(48, 20);
			this.tbTestInstanceCount.TabIndex = 11;
			this.tbTestInstanceCount.Text = "100";
			// 
			// btnGenerate
			// 
			this.btnGenerate.Location = new System.Drawing.Point(296, 208);
			this.btnGenerate.Name = "btnGenerate";
			this.btnGenerate.Size = new System.Drawing.Size(88, 24);
			this.btnGenerate.TabIndex = 11;
			this.btnGenerate.Text = "Generate";
			this.btnGenerate.Click += new System.EventHandler(this.btnGenerate_Click);
			// 
			// dgGeneratedTestInstances
			// 
			this.dgGeneratedTestInstances.AccessibleDescription = "Generated test instance sound clips and their coresponding numbers";
			this.dgGeneratedTestInstances.AccessibleRole = System.Windows.Forms.AccessibleRole.Table;
			this.dgGeneratedTestInstances.CaptionText = "Generated test instances";
			this.dgGeneratedTestInstances.DataMember = "";
			this.dgGeneratedTestInstances.HeaderForeColor = System.Drawing.SystemColors.ControlText;
			this.dgGeneratedTestInstances.Location = new System.Drawing.Point(8, 240);
			this.dgGeneratedTestInstances.Name = "dgGeneratedTestInstances";
			this.dgGeneratedTestInstances.ReadOnly = true;
			this.dgGeneratedTestInstances.Size = new System.Drawing.Size(464, 224);
			this.dgGeneratedTestInstances.TabIndex = 12;
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(16, 112);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(96, 16);
			this.label5.TabIndex = 16;
			this.label5.Text = "Change pitch by:";
			// 
			// tbPitchVariation
			// 
			this.tbPitchVariation.Location = new System.Drawing.Point(136, 110);
			this.tbPitchVariation.Name = "tbPitchVariation";
			this.tbPitchVariation.Size = new System.Drawing.Size(24, 20);
			this.tbPitchVariation.TabIndex = 17;
			this.tbPitchVariation.Text = "2";
			// 
			// label6
			// 
			this.label6.Location = new System.Drawing.Point(112, 112);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(24, 16);
			this.label6.TabIndex = 18;
			this.label6.Text = "+/-";
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem1});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem2});
			this.menuItem1.Text = "Help";
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 0;
			this.menuItem2.Text = "About";
			this.menuItem2.Click += new System.EventHandler(this.menuItem2_Click);
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(480, 470);
			this.Controls.Add(this.dgGeneratedTestInstances);
			this.Controls.Add(this.btnGenerate);
			this.Controls.Add(this.gbTestSettings);
			this.Controls.Add(this.button2);
			this.Controls.Add(this.btnExit);
			this.Controls.Add(this.btnSpeak);
			this.Controls.Add(this.txtSpeakText);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.Menu = this.mainMenu1;
			this.Name = "MainForm";
			this.Text = "Captcha Audio Generator";
			((System.ComponentModel.ISupportInitialize)(this.tbRate)).EndInit();
			this.gbTestSettings.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.dgGeneratedTestInstances)).EndInit();
			this.ResumeLayout(false);

		}
        #endregion

        /// <summary>
        ///     The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main() 
        {
            Application.Run(new MainForm());
        }


		/// <summary>
		/// Create a test instance string to speak.
		/// </summary>
		/// <param name="NumberCount">Numbers to generate. Max 3</param>
		/// <param name="MaxNumber">Size of largest number</param>
		/// <returns></returns>
		private ArrayList GetCaptchaText(int NumberCount, int MaxNumber)
		{
			Random rand = new Random();

			string StringToSpeak = String.Empty;			
			
			//Set mode of text (2 = "First number:".. 1 = "Here are the x numbers...", 3 = "123 55 1")
			int Mode = rand.Next(3);			

			ArrayList alResult = new ArrayList();

			//Generate numbers
			for(int Counter = 0; Counter < NumberCount; Counter++)
			{				
				switch (Mode)
				{
					case 1:
						if(Counter == 0)
						{
							StringToSpeak = Mode1Text.Replace("%s", NumberCount.ToString());
						} 
						break;
					case 2:
						switch (Counter)
						{
							case 0:
								StringToSpeak = Mode2Text1;
								break;
							case 1:
								StringToSpeak += Mode2Text2;
								break;
							case 2:
								StringToSpeak += Mode2Text3;
								break;
						}
						break;
					case 3:
						StringToSpeak += " ";
						break;
					default:
						break;
				}

				
				//Create a random number
				string TestNo = rand.Next(MaxNumber).ToString();

				//add number to result
				alResult.Add(TestNo);

				//Add the random number to the string to speak
				StringToSpeak += TestNo + ". ";
			}

			//add the string to speak
			alResult.Add(StringToSpeak);

			//return text to speak and numbers		
			return alResult;
		}





        private void btnSpeak_Click(object sender, System.EventArgs e)
        {
			//Create a TTS voice and speak.
			try 
			{
				SpeechVoiceSpeakFlags SpFlags = SpeechVoiceSpeakFlags.SVSFlagsAsync;
				SpVoice Voice = new SpVoice();

				//Set selected voice and speed
				Voice.Voice = Voice.GetVoices("","").Item(comboBox1.SelectedIndex);
				Voice.Rate = tbRate.Value;

				//Speak
				Voice.Speak(txtSpeakText.Text, SpFlags);
			}
			catch(Exception error)
			{
				MessageBox.Show("Speak error", "SimpleTTS", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}

        }

        
		
		/// <summary>
		/// Exit program.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnExit_Click(object sender, System.EventArgs e)
        {
		  this.Dispose();
        }



		/// <summary>
		/// Pick folder
		/// </summary>
		private void button1_Click(object sender, System.EventArgs e)
		{
			DialogResult result = folderBrowserDialog1.ShowDialog();
			if( result == DialogResult.OK )
			{
				lblTargetFolder.Text = folderBrowserDialog1.SelectedPath;
			}
		}

		/// <summary>
		/// Get sample text
		/// </summary>
		private void button2_Click(object sender, System.EventArgs e)
		{
			ArrayList alItems = GetCaptchaText(3, 100);
			txtSpeakText.Text = (string)alItems[alItems.Count-1];
		}


		/// <summary>
		/// Generate a random filename that does not exist in FolderPath.
		/// </summary>
		/// <param name="FolderPath">The folder to check</param>
		/// <returns>The name of a wav file</returns>
		private string GetRandomFileName(string FolderPath)
		{
			if(FolderPath.Length > 0)
			{
				Random rand = new Random();
				string FileName = rand.Next(999999999).ToString() + ".wav";
				
				//Make sure filename does not already exist in target dir
				while (File.Exists(FolderPath + "/" + FileName))
				{
					FileName = rand.Next(999999999).ToString() + ".wav";
				}

				return FileName;

			} 
			else 
			{
				//No folderpath
				return String.Empty;
			}

		}


		/// <summary>
		/// Start generating captcha sound files
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnGenerate_Click(object sender, System.EventArgs e)
		{

			//Make sure folder is selected
			if(lblTargetFolder.Text == SelectFolder)
			{
				//No folder selected
				MessageBox.Show ("Please select a target folder for the generated files", "Error", 
					MessageBoxButtons.OK, MessageBoxIcon.Warning);

				//Set focus to select button
				button1.Focus();

				return;
			}

			//Get number of tests to generate
			int TestCount = Int32.Parse(tbTestInstanceCount.Text);

			//Clear previously generated datatable items
			dgGeneratedTestInstances.DataSource = new DataTable();

			
			SpeechVoiceSpeakFlags SpFlags = SpeechVoiceSpeakFlags.SVSFlagsAsync;
			SpVoice Voice = new SpVoice();

			//Set selected voice and rate
			Voice.Voice = Voice.GetVoices("","").Item(comboBox1.SelectedIndex);
			Voice.Rate = tbRate.Value;

			Random rand = new Random();

			//Create test files
			for(int Counter = 0; Counter < TestCount; Counter++)
			{
				//Generate random filename
				string FileName = GetRandomFileName(lblTargetFolder.Text);

				//Stream audio to file

				//Get text to speak and numbers to verify (generate three random numbers from 0-100)
				ArrayList alCurrentTestInstance = GetCaptchaText(3, 100);

				SpeechStreamFileMode SpFileMode = SpeechStreamFileMode.SSFMCreateForWrite;

				SpFileStream SpFileStream = new SpFileStream();
				SpFileStream.Open(lblTargetFolder.Text + "/" + FileName, SpFileMode, false);


				//vary pitch randomly
				if(Int32.Parse(tbPitchVariation.Text) > 0)
				{
					int PitchDelta = rand.Next(0 - Int32.Parse(tbPitchVariation.Text), Int32.Parse(tbPitchVariation.Text));
					if(Voice.Rate + PitchDelta <= 10 && Voice.Rate + PitchDelta >= -10)
					{
						//Adjust rate
						Voice.Rate = Voice.Rate + PitchDelta;
					}
				}

				//Vary voice randomly
				if(comboBox1.SelectedIndex == 0)
				{
					//do not use the last limited domain voice in the list
					int VoiceIndex = rand.Next(0, Voice.GetVoices("","").Count - 1);
					Voice.Voice = Voice.GetVoices("","").Item(VoiceIndex);					
				}

				Voice.AudioOutputStream = SpFileStream;
				Voice.Speak((string)alCurrentTestInstance[alCurrentTestInstance.Count - 1], SpFlags);
				Voice.WaitUntilDone(Timeout.Infinite);

				SpFileStream.Close();

				//reset voice rate
				Voice.Rate = tbRate.Value;

				//Update result table
				DataRow drItem = dtItems.NewRow();

				drItem["FirstNo"] = alCurrentTestInstance[0];
				drItem["SecondNo"] = alCurrentTestInstance[1];
				drItem["ThirdNo"] = alCurrentTestInstance[2];
				drItem["FileName"] = FileName;
				drItem["Text"] = (string)alCurrentTestInstance[alCurrentTestInstance.Count - 1];
				drItem["Voice"] = Voice.Voice.GetDescription(0);

				dtItems.Rows.Add(drItem);
			}

			//Set result table
			dgGeneratedTestInstances.DataSource = dtItems;
		
		}

		private void menuItem2_Click(object sender, System.EventArgs e)
		{
			AboutForm Abfrm = new AboutForm();
			Abfrm.ShowDialog();
		}
    }
}
